/*
 * File: SoundDemo.java
 * --------------------
 * This program illustrates the use of the SoundClip class.
 */

// import acm.program.*;

import acm.gui.*;
import acm.program.*;
import acm.util.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import javax.swing.event.*;

/**
 * This program illustrates the use of the <code>SoundClip</code> class.
 */
public class SoundDemo extends ConsoleProgram implements ChangeListener {

	public void init() {
		add(createUserInterface(), SOUTH);
		addActionListeners();
		volumeSlider.addChangeListener(this);
	}

	private JPanel createUserInterface() {
		volumeSlider = new JSlider(0, 100, 100);
		volumeSlider.setMajorTickSpacing(100);
		volumeSlider.setMinorTickSpacing(25);
		volumeSlider.setPaintTicks(true);
		textField = new JTextField(TEXT_FIELD_SIZE);
		textField.addActionListener(this);
		textField.setActionCommand("Open");
		startButton = new JButton("Start");
		loopButton = new JButton("Loop");
		stopButton = new JButton("Stop");
		resetButton = new JButton("Reset");
		enableButtons(false);
		JPanel panel = new VPanel();
		JPanel row1 = new JPanel();
		JPanel row2 = new JPanel();
		JPanel row3 = new JPanel();
		row1.setLayout(new FlowLayout());
		row2.setLayout(new FlowLayout());
		row3.setLayout(new FlowLayout());
		row1.add(new JButton("JFK"));
		row1.add(new JButton("Browse"));
		row1.add(startButton);
		row1.add(loopButton);
		row1.add(stopButton);
		row1.add(resetButton);
		row2.add(new JLabel((isAppletMode()) ? "URL" : "File/URL"));
		row2.add(textField);
		row2.add(new JButton("Open"));
		row3.add(new JLabel("Volume:"));
		row3.add(volumeSlider);
		panel.add(row1);
		panel.add(row2);
		panel.add(row3);
		return panel;
	}

	public void actionPerformed(ActionEvent e) {
		try {
			String cmd = e.getActionCommand();
			if (cmd.equals("JFK")) {
				println("Loading JFK sample");
				setCurrentSound((SoundClip) MediaTools.loadAudioClip(SAMPLE_CLIP));
			} else if (cmd.equals("Browse")) {
				File file = chooseInputFile();
				println("Loading " + file.getName());
				if (file != null) setCurrentSound(new SoundClip(file));
			} else if (cmd.equals("Open")) {
				println("Loading " + textField.getText());
				setCurrentSound(new SoundClip(textField.getText()));
			} else if (cmd.equals("Start")) {
				if (sound != null) {
					sound.play();
				}
			} else if (cmd.equals("Loop")) {
				if (sound != null) {
					sound.loop();
				}
			} else if (cmd.equals("Stop")) {
				if (sound != null) sound.stop();
			} else if (cmd.equals("Reset")) {
				if (sound != null) sound.setFrameIndex(0);
			}
		} catch (Exception ex) {
			showErrorMessage(ex.getMessage());
			enableButtons(false);
		}
	}

/*
 * Monitors the volume slider and updates the volume of the sound.
 */
	public void stateChanged(ChangeEvent e) {
		if (sound == null) return;
		Object source = e.getSource();
		if (source == volumeSlider && sound != null) {
			sound.setVolume(volumeSlider.getValue() / 100.0);
		}
	}

/*
 * Sets the specified sound clip as the current sound.
 */
	private void setCurrentSound(SoundClip clip) {
		sound = clip;
		sound.setVolume(volumeSlider.getValue() / 100.0);
		String name = sound.toString();
		int paren = name.indexOf('(');
		if (paren >= 0) name = name.substring(paren);
		println(name);
		enableButtons(true);
	}

/*
 * Chooses a file for input using a file dialog.  If the user
 * clicks the Cancel button, the method returns null.  If
 * the user clicks Open, the method returns the File object.
 */
	private File chooseInputFile() {
		JFileChooser chooser = new JFileChooser();
		File cwd = new File(System.getProperty("user.dir"));
		File dir = new File(cwd, "sounds");
		if (!dir.isDirectory()) {
			File home = new File(System.getProperty("user.home"));
			dir = new File(home, "Sounds");
			if (!dir.isDirectory()) dir = new File(home, "Desktop/Sounds");
		}
		if (dir.isDirectory()) chooser.setCurrentDirectory(dir);
		chooser.setFileFilter(new FileChooserFilter(SOUND_PATTERN, "Sound files"));
		int result = chooser.showOpenDialog(this);
		if (result != JFileChooser.APPROVE_OPTION) return null;
		return chooser.getSelectedFile();
	}

/*
 * Enables or disables the four buttons used to start, stop, loop or reset
 * the clip.
 */
	private void enableButtons(boolean state) {
		startButton.setEnabled(state);
		loopButton.setEnabled(state);
		stopButton.setEnabled(state);
		resetButton.setEnabled(state);
	}

/* Private constants */
	private static final String SAMPLE_CLIP = "JFKMoonChallenge.wav";
	private static final String SOUND_PATTERN = "*.au;*.wav;*.aif;*.aiff";
	private static final int TEXT_FIELD_SIZE = 30;

/* Private instance variables */
	private JTextField textField;
	private JButton startButton;
	private JButton loopButton;
	private JButton stopButton;
	private JButton resetButton;
	private JSlider volumeSlider;
	private SoundClip sound;

/* Standard Java entry point */
/* This method can be eliminated in most Java environments */
	public static void main(String[] args) {
		new SoundDemo().start(args);
	}

}
